using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace Snake
{
	public enum CollisionType { Aim, Wall, Way }

	public class frmLevel : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Timer timer;
		private System.ComponentModel.IContainer components;
		private Point p;
		private System.Windows.Forms.PictureBox picture;
		private int iRichtung;
		private int iLevel = 1;
		private int iLives = 3;
		private int iBreite = 2;
		private string aimColor;
		private string wayColor;
		private string levelPass;
		private long lTicks = DateTime.Now.Ticks;
		private long lDiff;
		private long lInterval;
		private long lPoints = 0;

		public frmLevel()
		{
			this.Initialize(iLevel);
		}

		public frmLevel(int level)
		{
			this.Initialize(level);
		}

		private void Initialize(int level)
		{
			InitializeComponent();
			
			this.SetStyle(ControlStyles.UserPaint, true);
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.Text = "Level " + level;

			loadLevel(level);
			MessageBox.Show("Starting Level " + level + "\nLevel Pass: " + levelPass);
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.timer = new System.Windows.Forms.Timer(this.components);
			this.picture = new System.Windows.Forms.PictureBox();
			this.SuspendLayout();
			// 
			// timer
			// 
			this.timer.Interval = 20;
			this.timer.Tick += new System.EventHandler(this.timer_Tick);
			// 
			// picture
			// 
			this.picture.Name = "picture";
			this.picture.Size = new System.Drawing.Size(640, 480);
			this.picture.TabIndex = 0;
			this.picture.TabStop = false;
			this.picture.Paint += new System.Windows.Forms.PaintEventHandler(this.picture_Paint);
			// 
			// frmLevel
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(642, 479);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.picture});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "frmLevel";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Level #";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.frmMain_KeyDown);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.frmLevel_Closing);
			this.Load += new System.EventHandler(this.frmLevel_Load);
			this.ResumeLayout(false);

		}
		#endregion

		private void timer_Tick(object sender, System.EventArgs e)
		{
			lDiff = DateTime.Now.Ticks - lTicks;
			
			if(lDiff > lInterval)
			{
				Graphics gr = Graphics.FromImage(picture.Image);;

				p = this.getNewPosition(p, iRichtung);
				
				CollisionType ct = this.checkCollision(picture);
				if(ct == CollisionType.Wall)
				{
					timer.Enabled = false;
					MessageBox.Show("Wall :(", "Wall", MessageBoxButtons.OK, MessageBoxIcon.Information);
					iLives--;
					loadLevel(iLevel);
					timer.Enabled = true;
				}
				else if(ct == CollisionType.Aim)
				{
					timer.Enabled = false;
					MessageBox.Show("Goal :)", "Goal", MessageBoxButtons.OK, MessageBoxIcon.Information);
					iLevel++;
					iLives++;
					loadLevel(iLevel);
					MessageBox.Show("Starting Level " + iLevel + "\nLevel Pass: " + levelPass, "New Level", MessageBoxButtons.OK, MessageBoxIcon.Information);
					timer.Enabled = true;
				}
				else if(ct == CollisionType.Way)
				{
					gr.FillRectangle(new SolidBrush(Color.Blue), p.X, p.Y, iBreite, iBreite);
					gr.FillRectangle(new SolidBrush(Color.Red), p.X, p.Y, 1, 1);
				}

				picture.Refresh();
				lTicks = DateTime.Now.Ticks;
				lPoints++;
			}
		}

		private void frmMain_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			switch(e.KeyCode)
			{
				case Keys.Up:
					iRichtung = 0;
					break;
				case Keys.Right:
					iRichtung = 1;
					break;
				case Keys.Down:
					iRichtung = 2;
					break;
				case Keys.Left:
					iRichtung = 3;
					break;
			}
		}

		private Point getNewPosition(Point p, int iRichtung)
		{
			switch(iRichtung)
			{
				case 0:
					p.Y -= 1;
					break;
				case 1:
					p.X += 1;
					break;
				case 2:
					p.Y += 1;
					break;
				case 3:
					p.X -= 1;
					break;
			}

			return p;
		}

		private CollisionType checkCollision(PictureBox pb)
		{
			Color c;
			Bitmap b = (Bitmap) pb.Image;

			if(p.X < 0)
			{
				p.X = pb.Size.Width - 1;
			}
			else if(p.X + 2 > pb.Size.Width)
			{
				p.X = 0;
			}
			else if(p.Y < 0)
			{
				p.Y = pb.Size.Height - 1;
			}
			else if(p.Y + 2 > pb.Size.Height)
			{
				p.Y = 0;
			}

			if(iRichtung == 1 || iRichtung == 2)
			{
				c = b.GetPixel(p.X + (iBreite - 1), p.Y + (iBreite - 1));
			}
			else
			{
				c = b.GetPixel(p.X, p.Y);
			}

			if(c.Name == aimColor)
			{
				return CollisionType.Aim;
			}
			else if(c.Name == wayColor)
			{
				return CollisionType.Way;
			}

			return CollisionType.Wall;
		}

		private void loadLevel(int level)
		{
			if(iLives < 0)
			{
				MessageBox.Show("No more lives :(", "Dead", MessageBoxButtons.OK, MessageBoxIcon.Information);
				if(Highscores.IsHighScore(lPoints))
				{
					MessageBox.Show("New Highscore !", "Highscore", MessageBoxButtons.OK, MessageBoxIcon.Information);
					Highscores.SaveHighscore("Alex", lPoints);
				}
				this.Close();
				return;
			}

			if(File.Exists(level + ".bmp") && File.Exists(level + ".ini"))
			{
				StreamReader reader = new StreamReader(level + ".ini");
				iRichtung = Convert.ToInt32(reader.ReadLine());
				p = new Point(Convert.ToInt32(reader.ReadLine()), Convert.ToInt32(reader.ReadLine()));
				lInterval = Convert.ToInt32(reader.ReadLine());
				aimColor = reader.ReadLine();
				wayColor = reader.ReadLine();
				levelPass = reader.ReadLine();

				picture.Image = Image.FromFile(level + ".bmp");
			}
			else
			{
				MessageBox.Show("No more level", "End", MessageBoxButtons.OK, MessageBoxIcon.Information);
				this.Close();
			}
		}

		private void picture_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			e.Graphics.DrawString("Lives: " + iLives + " Points: " + lPoints, new Font("Arial", 12), new SolidBrush(Color.Red), 10, 10);picture.CreateGraphics().DrawString("Lives: ", new Font("Arial", 12), new SolidBrush(Color.Red), 10, 10);
		}

		private void frmLevel_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			timer.Enabled = false;
		}

		private void frmLevel_Load(object sender, System.EventArgs e)
		{
			timer.Enabled = true;
		}
	}
}